@@ -76,6 +76,7 @@
 #include <sepol/policydb/hierarchy.h>
 #include <sepol/policydb/flask.h>
 #include <sepol/policydb/expand.h>
+#include <sepol/policydb/link.h>
 
 #include "queue.h"
 #include "checkpolicy.h"
@@ -354,6 +355,17 @@ int change_bool(char *name, int state)
 	return 0;
 }
 
+static int check_level(hashtab_key_t key, hashtab_datum_t datum, void *arg)
+{
+	level_datum_t *levdatum = (level_datum_t *) datum;
+
+	if (!levdatum->isalias && !levdatum->defined) {
+		fprintf(stderr, "Error:  sensitivity %s was not used in a level definition!\n", key);
+		return -1;
+	}
+	return 0;
+}
+
 int main(int argc, char **argv)
 {
 	sepol_security_class_t tclass;
@@ -525,11 +537,20 @@ int main(int argc, char **argv)
 		if (policydb_errors) 
 			exit(1);
 
+		if (hashtab_map(policydbp->p_levels.table, check_level, NULL))
+			exit(1);
+
 		if (policydb_init(&policydb)) {
 			fprintf(stderr, "%s:  policydb_init failed\n", argv[0]);
 			exit(1);
 		}
 
+		/* Linking takes care of optional avrule blocks */
+        	if (link_modules(NULL, &parse_policy, NULL, 0, 0)) {
+			fprintf(stderr, "Error while resolving optionals\n");
+			exit(1);
+		}
+
 		if (expand_module(NULL, &parse_policy, &policydb, 0, 1)) {
                         fprintf(stderr, "Error while expanding policy\n");
 			exit(1);
