@@ -60,62 +60,24 @@ void usage(char *progname)
 	exit(1);
 }
 
-/* borrowed from checkpolicy.c */
-static int find_perm(hashtab_key_t key, hashtab_datum_t datum, void *p)
-{
-	unsigned int *valuep;
-	perm_datum_t *perdatum;
-
-	valuep = (unsigned int *) p;
-	perdatum = (perm_datum_t *) datum;
-
-	if (*valuep == perdatum->value)
-		return (int) key;
-
-	return 0;
-}
-
 static void render_access_mask(uint32_t mask, uint32_t class, policydb_t *p, FILE *fp)
 {
-	unsigned int i;
-	class_datum_t *cladatum;
 	char *perm;
-	cladatum = p->class_val_to_struct[class - 1];
 	fprintf(fp, "{");
-	for (i = 1; i <= sizeof(mask) * 8; i++) {
-		if (mask & (1 << (i - 1))) {
-			perm = (char *) hashtab_map(cladatum->permissions.table,
-				  find_perm, &i);
-
-			if (!perm && cladatum->comdatum) {
-				perm = (char *) hashtab_map(cladatum->comdatum->permissions.table,
-				  find_perm, &i);
-			}
-			if (perm)
-				fprintf(fp, " %s", perm);
-		}
-	}
-	fprintf(fp, " }");
+	perm = sepol_av_to_string(p, class, mask);
+	if (perm)
+		fprintf(fp, "%s ", perm);
+	fprintf(fp, "}");
 }
 
 static void render_access_bitmap(ebitmap_t *map, uint32_t class, policydb_t *p, FILE *fp)
 {
 	unsigned int i;
-        uint32_t perm_value;
-	class_datum_t *cladatum;
 	char *perm;
-	cladatum = p->class_val_to_struct[class - 1];
 	fprintf(fp, "{");
         for (i = ebitmap_startbit(map); i < ebitmap_length(map); i++) {
                 if (ebitmap_get_bit(map, i)) {
-                        perm_value = i + 1;
-			perm = (char *) hashtab_map(cladatum->permissions.table,
-				  find_perm, &perm_value);
-
-			if (!perm && cladatum->comdatum) {
-				perm = (char *) hashtab_map(cladatum->comdatum->permissions.table,
-				  find_perm, &perm_value);
-			}
+			perm = sepol_av_to_string(p, class, 1 << i);
 			if (perm)
 				fprintf(fp, " %s", perm);
 		}
@@ -303,11 +265,11 @@ int display_avrule(avrule_t *avrule, uint32_t what, policydb_t *policy, FILE *fp
 		fprintf(fp, " }");
 	fprintf(fp, " ");
 
-	if( avrule->specified & AVRULE_AV) {
+	if( avrule->specified & (AVRULE_AV | AVRULE_NEVERALLOW)) { 
 		render_access_mask(avrule->perms->data, avrule->perms->class, policy, fp);
 	} else if ( avrule->specified & AVRULE_TYPE) {
                 display_id(policy, fp, SYM_TYPES, avrule->perms->data - 1, "");
-	}
+	} 
 
 	fprintf(fp, ";\n");
 
